@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -400,6 +400,15 @@ complete_file(void)
 	DIR *dd;
 	static USTRING mbdir = UNULL;
 
+	if (rq.dirw == 0) {
+		/* special case: bare tilde */
+		if (wcscmp(rq.str,L"~") == 0) {
+			register_candidate(L"~",0,FT_DIRECTORY,0);
+			return;
+		}
+		rq.dir = ".";
+		rq.dirw = L".";
+	}
 	dir = rq.dir ? rq.dir : us_convert2mb(rq.dirw,&mbdir);
 	if ( (dd = opendir(dir)) == 0) {
 		compl.err = errno;
@@ -720,7 +729,7 @@ compl_name(int type)
 	rq.qflags = QFL_NONE;
 	rq.type = type;
 	rq.dir = 0;
-	rq.dirw = L".";
+	rq.dirw = 0;
 	rq.strlen = end - start;
 	rq.str = usw_copyn(&str_buff,pline + start,rq.strlen);
 
@@ -736,7 +745,7 @@ compl_name(int type)
 
 /*
  * compl_file() attempts to complete the partial text in the command line
- * type is on of the COMPL_TYPE_AUTO, CMD, DIR, DIRPANEL, HISTORY, FILE, FILTER
+ * type is on of the COMPL_TYPE_AUTO, CMD, DIR, DIRPANEL, HISTORY, FILE, or DRYRUN
  *
  * return value:
  *    0  = completion process completed (successfully or not)
@@ -869,13 +878,11 @@ compl_file(int type)
 	/* set rq.dirw */
 	if (pslash == 0) {
 		rq.str = pstart;
-		rq.dirw = L".";
-		rq.dir = ".";
+		rq.dirw = 0;
 	}
 	else {
 		rq.str = pslash + 1;
 		rq.dirw = pstart;
-		rq.dir = 0;	/* will be converted from rq.dirw on demand */
 
 		dirlen = pslash - pstart;
 		/* dequote 'dir' (if appropriate) + add terminating null byte */
@@ -894,6 +901,7 @@ compl_file(int type)
 		else if (tilde)
 			rq.dirw = dir_tilde(rq.dirw);
 	}
+	rq.dir = 0;	/* will be converted from rq.dirw on demand */
 
 	/* set the proper completion type */
 	if (type == COMPL_TYPE_DIRPANEL) {
