@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -24,6 +24,7 @@
 # include <curses.h>
 #endif
 #include <stdarg.h>			/* log.h */
+#include <stdio.h>			/* fputs() */
 #include <stdlib.h>			/* exit() */
 #include <wctype.h>			/* iswcntrl() */
 
@@ -47,6 +48,7 @@
 #include "notify.h"			/* notif_prepare() */
 #include "opt.h"			/* opt_save() */
 #include "panel.h"			/* cx_pan_xxx() */
+#include "preview.h"		/* preview_prepare() */
 #include "rename.h"			/* rename_prepare() */
 #include "select.h"			/* select_prepare() */
 #include "sort.h"			/* sort_prepare() */
@@ -92,11 +94,12 @@ typedef struct {
 #define OPT_ALL		4
 /*
  * OPT_CURS: call the handler function only if the cursor is on a valid regular line,
- * i.e. not on an extra line or in an empty panel
+ * i.e. not on an extra line and not in an empty panel
  *     extra panel lines:   curs < 0
  *     regular panel lines: curs >= 0
  * note1: in a panel with extra lines is OPT_CURS not necessary for the <enter> key
- *     (i.e. ctrl-M), see the EXTRA_LINE table
+ *     (i.e. ctrl-M), because when the cursor is not on a regular panel line, the
+ *     <enter> is handled by an EXTRA_LINE function
  * note2: see the warning at tab_mainmenu[] !
  *
  * OPT_NOFILT: ignore the binding when filtering is active
@@ -120,6 +123,7 @@ static CXM(log,LOG)
 static CXM(mainmenu,MAINMENU)
 static CXM(notif,NOTIF)
 static CXM(paste,PASTE)
+static CXM(preview,PREVIEW)
 static CXM(rename,RENAME)
 static CXM(select,SELECT)
 static CXM(sort,SORT)
@@ -175,6 +179,7 @@ static KEY_BINDING tab_cfg[] = {
 	{ 0, 0,  L's',			cx_cfg_default,		OPT_CURS	},
 	{ 0, 0,  L'o',			cx_cfg_original,	OPT_CURS	},
 	{ 0, 1,  L'c',			cx_cfg_noexit,		0			},
+	{ 0, 0,  WCH_CTRL('C'),	cx_cfg_noexit, 		0			},
 	END_TABLE
 };
 
@@ -194,17 +199,17 @@ static KEY_BINDING tab_cfg_menu[] = {
 };
 
 static KEY_BINDING tab_common[] = {
-	{ 0, 0,  WCH_CTRL('C'),	cx_trans_return,	0	},
-	{ 0, 0,  WCH_CTRL('F'),	cx_filter,			0	},
-	{ 0, 1,  L'c',			cx_mode_cfg,		0	},
-	{ 0, 1,  L'l',			cx_mode_log,		0	},
-	{ 0, 1,  L'n',			cx_mode_notif,		0	},
-	{ 0, 1,  L'o',			cx_mode_fopt,		0	},
-	{ 0, 1,  L'q',			cx_trans_quit,		0	},
-	{ 0, 1,  L'v',			cx_version,			0	},
-	{ 1, 0,  KEY_F(1),		cx_mode_help,		0	},
+	{ 0, 0,  WCH_CTRL('C'),	cx_trans_return,	0		},
+	{ 0, 0,  WCH_CTRL('F'),	cx_filter,			0		},
+	{ 0, 1,  L'c',			cx_mode_cfg,		0		},
+	{ 0, 1,  L'l',			cx_mode_log,		0		},
+	{ 0, 1,  L'n',			cx_mode_notif,		0		},
+	{ 0, 1,  L'o',			cx_mode_fopt,		0		},
+	{ 0, 1,  L'q',			cx_trans_quit,		0		},
+	{ 0, 1,  L'v',			cx_version,			0		},
+	{ 1, 0,  KEY_F(1),		cx_mode_help,		0		},
 #ifdef KEY_HELP
-	{ 1, 0,  KEY_HELP,		cx_mode_help,		0	},
+	{ 1, 0,  KEY_HELP,		cx_mode_help,		0		},
 #endif
 	END_TABLE
 };
@@ -284,6 +289,7 @@ static KEY_BINDING tab_editcmd[] = {
 	{ 0, 0,  WCH_CTRL('T'),	cx_select_toggle,	OPT_CURS	},
 	{ 0, 1,  WCH_CTRL('T'),	cx_select_range,	OPT_CURS	},
 	{ 0, 0,  WCH_CTRL('X'),	cx_files_xchg,		0			},
+	{ 0, 1,  L'e',			cx_mode_preview,	OPT_CURS	},
 	{ 0, 1,  L'g',			cx_mode_group,		0			},
 	{ 0, 1,  L'm',			cx_mode_mainmenu,	0			},
 	{ 0, 1,  L'p',			cx_complete_hist,	0			},
@@ -473,7 +479,7 @@ static KEY_BINDING tab_mainmenu2[] = {
 
 /* pseudo-table returned for mouse operations with OPT_ALL */
 static KEY_BINDING tab_mouse[] = {
-	{ 2, 0,  0,	noop,	0	},
+	{ 2, 0,  0,	cx_common_mouse,	0	},
 	END_TABLE
 };
 
@@ -535,6 +541,12 @@ static KEY_BINDING tab_pastemenu[] = {
 	END_TABLE
 };
 
+static KEY_BINDING tab_preview[] = {
+	{ 0, 0,  WCH_CTRL('M'),	cx_trans_return,	0		},
+	{ 2, 0,  0,				cx_preview_mouse,	OPT_ALL	},
+	END_TABLE
+};
+
 static KEY_BINDING tab_rename[] = {
 	{ 0, 0,  WCH_CTRL('M'),	cx_rename,	0	},
 	END_TABLE
@@ -595,7 +607,7 @@ static MODE_DEFINITION mode_definition[] = {
 		L"U/D = up/down, N = new, P = properties, <del> = remove",
 		bm_prepare, { tab_panel,tab_bm,0 } },
 	{ MODE_CFG, 0,
-		{ "cfg", "cfg_app", "cfg_cmd", "cfg_other" },
+		{ "cfg", "cfg_parameters" },
 		L"CONFIGURATION", L"<enter> = change, O = original, S = standard",
 		cfg_prepare, { tab_panel,tab_cfg,0 } },
 	{ MODE_CFG_EDIT_NUM, 0,
@@ -636,7 +648,7 @@ static MODE_DEFINITION mode_definition[] = {
 		dir_split_prepare, { tab_panel,tab_dir,0 } },
 	{ MODE_FILE, 0,
 		{ "file1", "file2", "file3" },
-		0, L"alt-M = main menu",
+		0, 0,
 		files_main_prepare, { tab_panel,tab_editcmd,tab_mainmenu,0 } },
 	{ MODE_FOPT, 1,
 		{ "filter_opt" },
@@ -675,10 +687,13 @@ static MODE_DEFINITION mode_definition[] = {
 		{ "paste" },
 		L"COMPLETE/INSERT NAME", 0,
 		paste_prepare, { tab_panel,tab_pastemenu,0 } },
+	{ MODE_PREVIEW, 0,
+		{ "preview" },
+		0,L"<enter> = close preview",
+		preview_prepare, { tab_panel,tab_preview,0 } },
 	{ MODE_RENAME, 0,
 		{ "rename" },
-		L"RENAME FILE",
-		L"This function is not intended for regular file renaming",
+		L"RENAME FILE", 0,
 		rename_prepare, { tab_rename,0 } },
 	{ MODE_SELECT, 0,
 		{ "select" },
@@ -856,22 +871,11 @@ do_action(wint_t key, KEY_BINDING **tables)
 
 	/* mouse event substitutions */
 	if (fkey == 2 && MI_DC(1)) {
-		if (MI_AREA(BAR)) {
-			key = KEY_F(1);
-			fkey = 1;	/* kinp.fkey is still 2 */
-		}
 		/* double click in a panel or on the input line -> enter */
-		else if (MI_CURSBAR || (MI_AREA(LINE) && textline)) {
+		if (MI_CURSBAR || (MI_AREA(LINE) && textline)) {
 			key = WCH_CTRL('M');
 			fkey = 0;	/* kinp.fkey is still 2 */
 		}
-		else if (MI_AREA(LINE) && textline == 0) {
-			if (get_current_mode() == MODE_CFG)
-				cx_cfg_noexit();
-			else
-				next_mode = MODE_SPECIAL_RETURN;
-			return tab_mouse;
-		}
 	}
 
 	key_lc = fkey != 0 ? key : towlower(key);
@@ -961,6 +965,7 @@ control_loop(int mode)
 			break;
 		win_sethelp(HELPMSG_BASE,clex_mode->modedef->help);
 		win_settitle(clex_mode->modedef->title);
+		win_bar();
 		if (panel != clex_mode->panel) {
 			if (panel->filtering || (clex_mode->panel && clex_mode->panel->filtering))
 				win_filter();
@@ -1036,6 +1041,7 @@ control_loop(int mode)
 	}
 
 	clex_mode = clex_mode->previous;
+	win_bar();
 	if (panel != clex_mode->panel) {
 		filter = panel->filtering || clex_mode->panel->filtering;
 		panel = clex_mode->panel;
@@ -1090,7 +1096,7 @@ cx_paste_pick(void)
 void
 cx_version(void)
 {
-	msgout(MSG_i,"Welcome to CLEX " VERSION CLEXRELEASE " !");
+	msgout(MSG_i,"Welcome to CLEX " VERSION " !");
 }
 
 /*
