@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -158,7 +158,7 @@ void
 cx_hist_next(void)
 {
 	if (pn_index == -1) {
-		msgout(MSG_i,"top of the history list");
+		msgout(MSG_i,"end of the history list (newest command)");
 		return;
 	}
 
@@ -175,7 +175,7 @@ void
 cx_hist_prev(void)
 {
 	if (pn_index >= hs_cnt - 1) {
-		msgout(MSG_i,"bottom of the history list");
+		msgout(MSG_i,"end of the history list (oldest command)");
 		return;
 	}
 
@@ -216,7 +216,7 @@ cx_hist_paste(void)
 void
 cx_hist_mouse(void)
 {
-	if (MI_B(3) && !MI_DRAG && MI_CURSBAR)
+	if (MI_PASTE)
 		cx_hist_paste();
 }
 
