@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -116,6 +116,8 @@ static struct {
 /* number of chars written to display lines, used for cursor position calculations */
 static int chars_in_line[MAX_CMDLINES];
 
+static wchar_t *bar;			/* win_bar() output */
+
 static void win_helpline(void);	/* defined below */
 static void win_position(void);	/* defined below */
 
@@ -183,8 +185,9 @@ void
 curses_initialize(void)
 {
 	int i;
-	const char *term;
-	static const char *compat[] = { "xterm","kterm","Eterm","dtterm","rxvt" };
+	const char *term, *astr;
+	static const char *compat[] = { "xterm","kterm","Eterm","dtterm","rxvt","aixterm" };
+	static const char *not_compat[] = { "ansi","vt","linux","dumb" };
 
 	if (disp_data.wait)
 		tty_press_enter();
@@ -217,14 +220,24 @@ curses_initialize(void)
 
 	disp_data.bs177 = key_backspace && strcmp(key_backspace,"\177") == 0;
 
-	if ( (term = getenv("TERM")) )
+	astr = "not known";
+	if ( (term = getenv("TERM")) ) {
 		for (i = 0; i < ARRAY_SIZE(compat); i++)
 			if (strncmp(term,compat[i],strlen(compat[i])) == 0) {
 				disp_data.xterm = 1;
+				astr = "yes";
 				break;
 			}
-	msgout(MSG_DEBUG,"Terminal type: \"%s\", xterm compatible: %s",
-	  term ? term : "undefined", disp_data.xterm ? "yes" : "don't know");
+		if (!disp_data.xterm)
+				for (i = 0; i < ARRAY_SIZE(not_compat); i++)
+					if (strncmp(term,not_compat[i],strlen(not_compat[i])) == 0) {
+						disp_data.noxterm = 1;
+						astr = "no";
+						break;
+					}
+	}
+	msgout(MSG_DEBUG,"Terminal type: \"%s\", ability to change the window title: \"%s\"",
+		term ? term : "undefined", astr);
 
 	disp_data.xwin = getenv("WINDOWID") && getenv("DISPLAY");
 	msgout(MSG_DEBUG,"X Window: %s",
@@ -308,7 +321,7 @@ screen_refresh(void)
 
 /* which screen area belongs the line 'ln' to ? */
 static int
-screen_area(int ln)
+screen_area(int ln, int col)
 {
 	if (ln < 0 || ln >= disp_data.scrlines)
 		return -1;
@@ -326,6 +339,8 @@ screen_area(int ln)
 		return AREA_HELP;
 	if (ln == 3)
 		return AREA_BAR;
+	if (ln == 4 && textline && col < textline->promptwidth)
+		return AREA_PROMPT;
 	return AREA_LINE;
 }
 
@@ -385,7 +400,7 @@ scr2curs_help(int y, int x)
 	HELP_LINE *ph;
 
 	curs = panel->top + y;
-	if (curs < 0 && curs >= panel->cnt)
+	if (curs < 0 || curs >= panel->cnt)
 		return -1;
 
 	ph = panel_help.line[curs];
@@ -403,6 +418,25 @@ scr2curs_help(int y, int x)
 	return i;
 }
 
+/* note: single width characters assumed */
+static int
+scr2curs_bar(int x)
+{
+	int curs;
+
+	x--;	/* make 0-based */
+	if (x < 0 || x >= wcslen(bar) || bar[x] == L' ')
+		return -1;
+
+	for (curs = 0; --x > 0; ) {
+		if (bar[x] == L'|')
+			return -1;
+		if (bar[x] == L' ' && bar[x-1] != L' ')
+			curs++;
+	}
+	return curs;
+}
+
 /* read the mouse tracking data */
 static int
 mouse_data(void)
@@ -438,7 +472,7 @@ mouse_data(void)
 	}
 
 	/* Y and X coordinates */
-	if ( (area = screen_area(my)) < 0)
+	if ( (area = screen_area(my, mx)) < 0)
 		return -1;
 	if ((xpos = 100 * mx / (disp_data.scrcols - 1)) > 100 )
 		return -1;
@@ -453,7 +487,7 @@ mouse_data(void)
 	minp.x = mx;
 	minp.y = my;
 	minp.motion = mstat & 0x20;	/* mouse is in motion */
-	minp.doubleclick =  click && minp.button == miprev.button && !minp.motion
+	minp.doubleclick = !miprev.doubleclick && click && minp.button == miprev.button && !minp.motion
 	  && minp.x == miprev.x && minp.y == miprev.y && now.tv_sec - prev.tv_sec < 2
 	  && 1000 * (int)(now.tv_sec - prev.tv_sec) + (int)(now.tv_usec - prev.tv_usec) / 1000
 		<= cfg_num(CFG_DOUBLE_CLICK);
@@ -469,6 +503,9 @@ mouse_data(void)
 				/* input line cursor */
 				minp.cursor = scr2curs(minp.y - LNO_EDIT,minp.x);
 			break;
+		case AREA_BAR:
+			minp.cursor = scr2curs_bar(minp.x);
+			break;
 		case AREA_BOTTOMFRAME:
 			if (panel->filter)
 				/* filter expression cursor */
@@ -500,11 +537,6 @@ kbd_rawkey(void)
 			if (--retries < 0)
 				err_exit("Cannot read the keyboard input");
 			type = get_wch(&kinp.key);
-
-			/* Fedora10 needs this nonsense to make the KEY_RESIZE work */
-			if (type == ERR)
-				wrefresh(curscr);
-
 		} while (type == ERR || kinp.key == WEOF);
 		if (type == KEY_CODE_YES) {
 #ifdef KEY_MOUSE
@@ -877,13 +909,13 @@ win_frame(void)
 static void
 twodirs(void)
 {
-	int w1, w2, opt1, opt2, width;
+	int w1, w2, rw1, opt1, opt2, width;
 	const wchar_t *dir1, *dir2;
 
 	/* directory names are separated by 2 spaces; their width is kept in ratio 5:3 */
 	width = disp_data.scrcols - 2;			/* available space */
 	dir1 = USTR(ppanel_file->dirw);
-	w1 = wc_cols(dir1,0,-1);
+	rw1 = w1 = wc_cols(dir1,0,-1);
 	dir2 = USTR(ppanel_file->other->dirw);
 	w2 = wc_cols(dir2,0,-1);
 	opt1 = opt2 = 0;
@@ -907,6 +939,9 @@ twodirs(void)
 	attroff(attrb);
 	addstr("  ");
 	putwcs_trunc(dir2,w2,opt2);
+
+	disp_data.dir1end = w1 < rw1 ? w1 : rw1;
+	disp_data.dir2start = w1 + 2;			/* exactly +3, but +2 feels more comfortable */
 }
 
 /* panel title - top screen line */
@@ -928,6 +963,12 @@ win_title(void)
 		putwcs_trunc_col(panel_help.title,disp_data.scrcols,0);
 		attroff(attrb);
 		break;
+	case MODE_PREVIEW:
+		addstr(" PREVIEW: ");
+		attron(attrb);
+		putwcs_trunc_col(panel_preview.title,disp_data.scrcols,0);
+		attroff(attrb);
+		break;
 	default:
 		/* static title */
 		addch(' ');
@@ -1125,6 +1166,12 @@ print_fields(FILE_ENTRY *pfe, int width, const wchar_t *fields)
 				fw = disp_data.date_len;
 				wtxt = pfe->mtime_str;
 				break;
+			case L'g':	/* file age */
+				fw = FE_AGE_STR - 1 - ppanel_file->cw_age;
+				txt = pfe->age_str;
+				if (txt[0])
+					txt += ppanel_file->cw_age;
+				break;
 			case L'i':	/* inode change date/time */
 				fw = disp_data.date_len;
 				wtxt = pfe->ctime_str;
@@ -1259,7 +1306,7 @@ win_infoline(void)
 	int curs;
 
 	move(LNO_INFO,0);
-	addstr("  ");
+	addstr("  ");		/* MARGIN2 */
 
 	/* extra panel lines */
 	if (panel->curs < 0 && panel->min < 0 && !panel->filtering
@@ -1293,7 +1340,7 @@ win_infoline(void)
 			break;
 		case PANEL_TYPE_LOG:
 			putwcs_trunc(convert2w(panel_log.line[curs]->levelstr),16,0);
-			putwcs_trunc_col(convert2w(panel_log.line[curs]->timestamp),disp_data.scrcols,0);
+			putwcs_trunc_col(convert2w(panel_log.line[curs]->timestamp),disp_data.scrcols - MARGIN2,0);
 			break;
 		case PANEL_TYPE_CMP:
 			if (curs < ARRAY_SIZE(info_cmp))
@@ -1397,6 +1444,7 @@ win_sethelp(int type, const wchar_t *msg)
 void
 win_bar(void)
 {
+	int pad;
 	static int len = 0;
 
 	if (len == 0)
@@ -1405,10 +1453,26 @@ win_bar(void)
 
 	attron(attrr);
 	move(LNO_BAR,0);
-	putwcs_trunc(L" CLEX file manager - press F1 for help ",disp_data.scrcols - len,0);
-	putwcs_trunc(user_data.loginw,len,OPT_NOPAD);
-	addch('@');
-	putwcs_trunc_col(user_data.hostw,disp_data.scrcols,0);
+
+	switch (get_current_mode()) {
+	case MODE_FILE:
+		bar = L" F1=help  alt-M=menu  |      CLEX file manager " ;
+		break;
+	case MODE_HELP:
+		bar = L" F1=help  ctrl-C=exit  <-- | CLEX file manager ";
+		break;
+	default:
+		bar = L" F1=help  ctrl-C=exit |      CLEX file manager ";
+	}
+	pad = putwcs_trunc_col(bar, disp_data.scrcols,OPT_NOPAD) - len;
+	if (pad < 0)
+		char_line(' ',len + pad);
+	else {
+		BLANK(pad);
+		putwcs_trunc(user_data.loginw,len,OPT_NOPAD);
+		addch('@');
+		putwcs_trunc_col(user_data.hostw,disp_data.scrcols,0);
+	}
 	attroff(attrr);
 }
 
@@ -1829,6 +1893,19 @@ draw_line_paste(int ln)
 	putwcs_trunc_col(description[ln],disp_data.panrcol,0);
 }
 
+void
+draw_line_preview(int ln)
+{
+	if (ln >= panel_preview.realcnt) {
+		attron(attrb);
+		putwcs_trunc(L" --- end of preview ---",disp_data.pancols,0);
+		attroff(attrb);
+		return;
+	}
+		
+	putwcs_trunc(USTR(panel_preview.line[ln]),disp_data.pancols,0);
+}
+
 void
 draw_line_sort(int ln)
 {
