@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -20,6 +20,7 @@
 #include "mouse.h"
 
 #include "cfg.h"		/* cfg_num() */
+#include "control.h"	/* control_loop() */
 #include "log.h"		/* msgout() */
 
 static FLAG enabled = 0;
@@ -63,3 +64,19 @@ mouse_restore(void)
 		fflush(stdout);
 	}
 }
+
+void
+cx_common_mouse(void)
+{
+	if (MI_B(2)) {
+		msgout(MSG_i,"press the shift if you want to paste or copy text with the mouse");
+		return;
+	}
+
+	if (MI_AREA(BAR) && MI_DC(1)) {
+		if (minp.cursor == 0)
+			control_loop(MODE_HELP);
+		else if (minp.cursor == 1)
+			next_mode = MODE_SPECIAL_RETURN;
+	}
+}
