@@ -2,7 +2,7 @@
  *
  * CLEX File Manager
  *
- * Copyright (C) 2001-2009 Vlado Potisk <vlado_potisk@clex.sk>
+ * Copyright (C) 2001-2011 Vlado Potisk <vlado_potisk@clex.sk>
  *
  * CLEX is free software without warranty of any kind; see the
  * GNU General Public License as set out in the "COPYING" document
@@ -304,7 +304,7 @@ userdata_initialize(void)
 
 	user_data.nowrite = 0;
 
-	msgout(MSG_AUDIT,"CLEX version: \""VERSION CLEXRELEASE "\"");
+	msgout(MSG_AUDIT,"CLEX version: \""VERSION "\"");
 	msgout(MSG_HEADING,"Examining data of your account");
 
 	myuid = getuid();
@@ -691,7 +691,7 @@ cx_user_paste(void)
 void
 cx_user_mouse(void)
 {
-	if (MI_B(3) && !MI_DRAG && MI_CURSBAR)
+	if (MI_PASTE)
 		cx_user_paste();
 }
 
@@ -751,6 +751,6 @@ cx_group_paste(void)
 void
 cx_group_mouse(void)
 {
-	if (MI_B(3) && !MI_DRAG && MI_CURSBAR)
+	if (MI_PASTE)
 		cx_group_paste();
 }
